﻿using Chapter15.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Data.Xml.Dom;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Notifications;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

namespace Chapter15
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class MainPage : Page
    {

        private NavigationHelper navigationHelper;
        private ObservableDictionary defaultViewModel = new ObservableDictionary();

        /// <summary>
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }

        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }


        public MainPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
            this.navigationHelper.SaveState += navigationHelper_SaveState;
        }

        /// <summary>
        /// Populates the page with content passed during navigation. Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session. The state will be null the first time a page is visited.</param>
        private void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/></param>
        /// <param name="e">Event data that provides an empty dictionary to be populated with
        /// serializable state.</param>
        private void navigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedFrom(e);
        }

        #endregion

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(BasicPage1));
        }

        //Update tile with message
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            XmlDocument wideTile =
                TileUpdateManager.GetTemplateContent(TileTemplateType.TileWide310x150ImageAndText02);
            XmlNodeList textNodes = wideTile.GetElementsByTagName("text");
            textNodes[0].InnerText = "Your author has something to say.";
            textNodes[1].InnerText = "Again.";
            XmlNodeList imageNodes = wideTile.GetElementsByTagName("image");
            ((XmlElement)imageNodes[0]).SetAttribute("src", "ms-appx:///Assets/BookLogoWide.png");
            ((XmlElement)imageNodes[0]).SetAttribute("alt", "your author");
            XmlNodeList binding = wideTile.GetElementsByTagName("binding");
            ((XmlElement)binding[0]).SetAttribute("branding", "logo");

            XmlDocument squareTile =
                TileUpdateManager.GetTemplateContent(TileTemplateType.TileSquare150x150PeekImageAndText04);
            textNodes = squareTile.GetElementsByTagName("text");
            textNodes[0].InnerText = "Your author has something to say. Again.";
            imageNodes = squareTile.GetElementsByTagName("image");
            ((XmlElement)imageNodes[0]).SetAttribute("src", "ms-appx:///Assets/Logo.png");
            ((XmlElement)imageNodes[0]).SetAttribute("alt", "your author");
            binding = squareTile.GetElementsByTagName("binding");
            ((XmlElement)binding[0]).SetAttribute("branding", "none");

            IXmlNode node = wideTile.ImportNode(binding[0], true);
            wideTile.GetElementsByTagName("visual").Item(0).AppendChild(node);
            TileNotification tn = new TileNotification(wideTile);
            TileUpdateManager.CreateTileUpdaterForApplication().Update(tn);
        }

        //Send Badge to tile
        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            XmlDocument badgeXml =
                BadgeUpdateManager.GetTemplateContent(BadgeTemplateType.BadgeGlyph);
            XmlElement badgeElement = (XmlElement)badgeXml.SelectSingleNode("/badge");
            badgeElement.SetAttribute("value", "alert");
            BadgeNotification bn = new BadgeNotification(badgeXml);
            BadgeUpdateManager.CreateBadgeUpdaterForApplication().Update(bn);
        }


        //Send Notification
        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            XmlDocument toastXml =
                ToastNotificationManager.GetTemplateContent(ToastTemplateType.ToastText01);
            XmlNodeList textNodes = toastXml.GetElementsByTagName("text");
            textNodes[0].InnerText = "Your author has something to say. Again.";
            ToastNotification tn = new ToastNotification(toastXml);
            //      ToastNotificationManager.CreateToastNotifier().Show(tn);

            //IXmlNode toastNode = toastXml.SelectSingleNode("/toast");
            //XmlElement audio = toastXml.CreateElement("audio");
            //audio.SetAttribute("src", "ms-winsoundevent:Notification.IM");
            //audio.SetAttribute("loop", "false");
            //toastNode.AppendChild(audio);

            ToastNotificationManager.CreateToastNotifier().Show(tn);
        }
    }
}
