﻿
' TODO: Connect the Search Results Page to your in-app search.
' The Search Results Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234240

''' <summary>
''' This page displays search results when a global search is directed to this application.
''' </summary>
Public NotInheritable Class ColorSearchResults
    Inherits Page

    Private _navigationHelper As Common.NavigationHelper

    Private currentColors As IList(Of ColorDetails)

    Public ReadOnly Property NavigationHelper() As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    ''' <summary>
    ''' This can be changed to a strongly typed view model.
    ''' </summary>
    Public ReadOnly Property DefaultViewModel As Common.ObservableDictionary
        Get
            Return Me._defaultViewModel
        End Get
    End Property
    Private _defaultViewModel As New Common.ObservableDictionary()


    Public Sub New()
        Me.InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)
        AddHandler Me._navigationHelper.LoadState, AddressOf NavigationHelper_LoadState
    End Sub

    
    Private Sub NavigationHelper_LoadState(sender As Object, e As Common.LoadStateEventArgs)
        Dim queryText As String = TryCast(e.NavigationParameter, [String])

        'Dim filterList As New List(Of Filter)()
        'filterList.Add(New Filter("All", 0, True))

        Dim allColors As IList(Of ColorDetails) = (New ColorCollection()).ToList()
        If [String].IsNullOrEmpty(queryText) Then
            currentColors = allColors
        Else
            Dim results = From c In allColors Where c.ColorName.StartsWith(queryText, StringComparison.CurrentCultureIgnoreCase) Select c
            currentColors = results.ToList()
        End If
        Me.DefaultViewModel("Results") = currentColors
        Dim filterList = New List(Of Filter)()
        filterList.Add(New Filter("All", currentColors.Count, True))
        Dim RedCount As Integer = (From c In currentColors Where c.ColorGroup = ColorGroup.Red Select c).Count()
        filterList.Add(New Filter("Red", RedCount, False))

        Dim GreenCount As Integer = (From c In currentColors Where c.ColorGroup = ColorGroup.Green Select c).Count()
        filterList.Add(New Filter("Green", GreenCount, False))

        Dim BlueCount As Integer = (From c In currentColors Where c.ColorGroup = ColorGroup.Blue Select c).Count()
        filterList.Add(New Filter("Blue", BlueCount, False))


        ' Communicate results through the view model
        Me.DefaultViewModel("QueryText") = ChrW(&H201C) + queryText + ChrW(&H201D)
        Me.DefaultViewModel("Filters") = filterList
        Me.DefaultViewModel("ShowFilters") = filterList.Count > 1
    End Sub

    
    Private Sub Filter_Checked(sender As Object, e As RoutedEventArgs)
        Dim filter As Object = TryCast(sender, FrameworkElement).DataContext

        ' Mirror the change into the CollectionViewSource.
        If filtersViewSource.View IsNot Nothing Then
            filtersViewSource.View.MoveCurrentTo(filter)
        End If

        ' Determine what filter was selected
        Dim selectedFilter As Filter = TryCast(filter, Filter)
        If selectedFilter IsNot Nothing Then
            ' Mirror the results into the corresponding Filter object to allow the
            ' RadioButton representation used when not snapped to reflect the change
            selectedFilter.Active = True

            ' Respond to the change in active filter by setting Me.DefaultViewModel["Results"]
            '       to a collection of items with bindable Image, Title, Subtitle, and Description properties
            If selectedFilter.Name = "All" Then
                Me.DefaultViewModel("Results") = currentColors
            Else
                Dim filteredResults = From c In currentColors Where c.ColorGroup.ToString() = selectedFilter.Name Select c
                Me.DefaultViewModel("Results") = filteredResults.ToList()
            End If

            ' Ensure results are found
            Dim results As Object = Nothing

            If Me.DefaultViewModel.TryGetValue("Results", results) Then
                Dim ResultsCollection As ICollection = TryCast(results, ICollection)
                If ResultsCollection IsNot Nothing AndAlso ResultsCollection.Count <> 0 Then
                    VisualStateManager.GoToState(Me, "ResultsFound", True)
                    Return
                End If
            End If
        End If
        ' Display informational text when there are no search results.
        VisualStateManager.GoToState(Me, "NoResultsFound", True)
    End Sub

#Region "NavigationHelper registration"

    ''' The methods provided in this section are simply used to allow
    ''' NavigationHelper to respond to the page's navigation methods.
    ''' 
    ''' Page specific logic should be placed in event handlers for the  
    ''' <see cref="Common.NavigationHelper.LoadState"/>
    ''' and <see cref="Common.NavigationHelper.SaveState"/>.
    ''' The navigation parameter is available in the LoadState method 
    ''' in addition to page state preserved during an earlier session.

    Protected Overrides Sub OnNavigatedTo(e As NavigationEventArgs)
        _navigationHelper.OnNavigatedTo(e)
    End Sub

    Protected Overrides Sub OnNavigatedFrom(e As NavigationEventArgs)
        _navigationHelper.OnNavigatedFrom(e)
    End Sub

#End Region

    ''' <summary>
    ''' View model describing one of the filters available for viewing search results.
    ''' </summary>
    Private NotInheritable Class Filter
        Implements INotifyPropertyChanged
        Private _name As String
        Private _count As Integer
        Private _active As Boolean

        Public Sub New(name As String, count As Integer, Optional active As Boolean = False)
            Me.Name = name
            Me.Count = count
            Me.Active = active
        End Sub

        Public Overrides Function ToString() As String
            Return Description
        End Function

        Public Property Name As String
            Get
                Return _name
            End Get
            Set(value As String)
                If Me.SetProperty(_name, value) Then Me.OnPropertyChanged("Description")
            End Set
        End Property

        Public Property Count As Integer
            Get
                Return _count
            End Get
            Set(value As Integer)
                If Me.SetProperty(_count, value) Then Me.OnPropertyChanged("Description")
            End Set
        End Property

        Public Property Active As Boolean
            Get
                Return _active
            End Get
            Set(value As Boolean)
                Me.SetProperty(_active, value)
            End Set
        End Property

        Public ReadOnly Property Description As String
            Get
                Return String.Format("{0} ({1})", _name, _count)
            End Get
        End Property

        ''' <summary>
        ''' Multicast event for property change notifications.
        ''' </summary>
        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        ''' <summary>
        ''' Checks if a property already matches a desired value.  Sets the property and
        ''' notifies listeners only when necessary.
        ''' </summary>
        ''' <typeparam name="T">Type of the property.</typeparam>
        ''' <param name="storage">Reference to a property with both getter and setter.</param>
        ''' <param name="value">Desired value for the property.</param>
        ''' <param name="propertyName">Name of the property used to notify listeners.  This
        ''' value is optional and can be provided automatically when invoked from compilers that
        ''' support CallerMemberName.</param>
        ''' <returns>True if the value was changed, false if the existing value matched the
        ''' desired value.</returns>
        Private Function SetProperty(Of T)(ByRef storage As T, value As T, <CallerMemberName> Optional propertyName As [String] = Nothing) As Boolean
            If Object.Equals(storage, value) Then
                Return False
            End If

            storage = value
            Me.OnPropertyChanged(propertyName)
            Return True
        End Function

        ''' <summary>
        ''' Notifies listeners that a property value has changed.
        ''' </summary>
        ''' <param name="propertyName">Name of the property used to notify listeners.  This
        ''' value is optional and can be provided automatically when invoked from compilers
        ''' that support <see cref="CallerMemberNameAttribute"/>.</param>
        Private Sub OnPropertyChanged(<CallerMemberName> Optional propertyName As String = Nothing)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
        End Sub

    End Class

    Private Sub resultsGridView_ItemClick(sender As Object, e As ItemClickEventArgs) Handles resultsGridView.ItemClick
        Dim color As ColorDetails = DirectCast(e.ClickedItem, ColorDetails)
        Me.Frame.Navigate(GetType(ColorItem), color.ColorName)
    End Sub
End Class
